//////////////////////////////////////////////////////////////////////////
//
// pgAgent - PostgreSQL Tools
// 
// Copyright (C) 2002 - 2012, The pgAdmin Development Team
// This software is released under the PostgreSQL Licence
//
// pgaevent.rc - win32 Resources
//
//////////////////////////////////////////////////////////////////////////

LANGUAGE 0x9,0x1
1 11 MSG00001.bin

#include <winver.h>

// Icon (Don't remove the aaa prefix - it makes it the default icon)
aaaPGAEVENT      ICON    DISCARDABLE     "../include/pgagent.ico"


VS_VERSION_INFO VERSIONINFO 
FILEVERSION    3,2,1,0
PRODUCTVERSION 3,2,1,0
FILEOS         VOS__WINDOWS32
FILETYPE       VFT_APP
BEGIN
    BLOCK "StringFileInfo"
    BEGIN
        BLOCK "040904E4"
        BEGIN 
            VALUE "FileVersion",     "3.2.1", "\0"
            VALUE "File Version",    "3.2.1", "\0"
            VALUE "FileDescription", "pgaevent - pgAgent Event Log Message DLL", "\0"
            VALUE "LegalCopyright",  "\251 2002 - 2012, The pgAdmin Development Team", "\0"
            VALUE "LegalTrademarks", "This software is released under the PostgreSQL Licence.", "\0"
            VALUE "InternalName",    "pgaevent", "\0"
            VALUE "OriginalFilename","pgaevent.dll", "\0"
            VALUE "ProductName",     "pgAgent", "\0"
            VALUE "ProductVersion",  "3.2.1", "\0"
        END
    END
    BLOCK "VarFileInfo" 
    BEGIN 
        VALUE "Translation", 0x0409, 0x04E4
    END 
END 



